/*
	For use with SQL Server 2016 CTP3
	Enable Query Store
	
	Expand AdventureWorks2016 database first,
	don't see anything with QueryStore

	Enable through the UI
*/

/*
	this is the default if you script it out
*/
USE [master];
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE = ON;
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE (
	OPERATION_MODE = READ_WRITE
	);
GO


/*
	this is what it's really setting
*/
USE [master]
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE = ON
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE (
	OPERATION_MODE = READ_WRITE, 
	CLEANUP_POLICY = (STALE_QUERY_THRESHOLD_DAYS = 367), 
	DATA_FLUSH_INTERVAL_SECONDS = 900,  
	INTERVAL_LENGTH_MINUTES = 60, 
	MAX_STORAGE_SIZE_MB = 100, 
	QUERY_CAPTURE_MODE = ALL, 
	SIZE_BASED_CLEANUP_MODE = OFF, 
	MAX_PLANS_PER_QUERY = 200)
GO



USE [master];
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE (
	OPERATION_MODE = READ_WRITE,
		/*	READ_WRITE = data collection; READ_ONLY = no data collection */
	CLEANUP_POLICY = (STALE_QUERY_THRESHOLD_DAYS = 30),  
		/*	how long to retain data in the query store, default is 367, set it something smaller */
	DATA_FLUSH_INTERVAL_SECONDS = 60,
		/*	how frequently QS data is written to disk, default is 900 (15 minutes) 
			how much QS data are you willing to lose in case of server crash
		*/
	INTERVAL_LENGTH_MINUTES = 1, 
		/*	interval across which QS data is aggregated, default is 60 minutes
			use a smaller value if you need finer granularity or less time to detect and mitigate issues
			this will affect the size of the data (lower value = more data) 
			From BOL: Note that arbitrary values are not allowed - 
			you should use one of the following: 1, 5, 10, 15, 30, 60, and 1440 minutes) 
		*/
	MAX_STORAGE_SIZE_MB = 1024,
		/*	size of the QS (when the max is hit, status changes to READ_ONLY) 
			default is 100MB...what this needs to be depends on your workload 
			and how much information you want to keep
		*/
	QUERY_CAPTURE_MODE = ALL,
		/* 
			are ALL queries captured, or only "relevant" ones? 
			ALL is default
			AUTO is infrequent queries and those with insignificant compile and execution duration are ignored
			the thresholds (execution count, compile and runtime duration) are internally determined
			Q: do we know what those are?
		*/
	SIZE_BASED_CLEANUP_MODE = OFF,
		/* 
			Do you want QS to go into clean up mode as usage gets close to MAX_STORAGE_SIZE_MB? 
			default is OFF
			it is recommended to activate this so that QS always run in read-write mode and is collecting data
		*/
	MAX_PLANS_PER_QUERY = 200);
		/*
			max number of plans to store for a query
		*/
GO


/*
	start up the DEMO2 scripts
*/
USE [AdventureWorks2016];
GO

/*
	check current status
*/
SELECT actual_state_desc, readonly_reason, desired_state_desc, current_storage_size_mb, 
    max_storage_size_mb, flush_interval_seconds, interval_length_minutes, 
    stale_query_threshold_days, size_based_cleanup_mode_desc, 
    query_capture_mode_desc, max_plans_per_query
FROM sys.database_query_store_options;


/*
	What do we see in QS?
*/
USE [AdventureWorks2016];
GO

SELECT *
FROM sys.query_store_query_text;
GO

SELECT *
FROM sys.query_store_query;
GO

SELECT 
	qst.query_text_id, qsq.query_id, qsq.object_id, 
	qsq.count_compiles, qst.query_sql_text, qst.statement_sql_handle, qsq.query_hash
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id


SELECT *
FROM sys.query_store_plan;


SELECT 
	qst.query_text_id, qsq.query_id, qsp.plan_id, qsq.object_id, 
	qsq.count_compiles, qsp.engine_version, qsp.compatibility_level, qsp.is_parallel_plan, 
	qsp.is_forced_plan,	qsp.is_trivial_plan, qsp.last_compile_duration, qsp.last_execution_time,
	qst.statement_sql_handle, qsq.query_hash, qsp.query_plan_hash, qst.query_sql_text, qsp.query_plan
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id
JOIN sys.query_store_plan qsp ON qsq.query_id = qsp.query_id;


SELECT p.plan_id, CAST(p.query_plan AS XML)
FROM sys.query_store_plan p;


/*
	Textual matching still in effect
*/
SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [Color];


SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [color];


SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY 
[Color];


SELECT 
	qst.query_text_id, qsq.query_id, qsp.plan_id, qsq.is_internal_query, qsq.object_id, 
	qsq.count_compiles, qsp.engine_version, qsp.compatibility_level, qsp.is_parallel_plan, 
	qsp.is_forced_plan,	qsp.is_trivial_plan, qsp.last_compile_duration, qsp.last_execution_time,
	qst.statement_sql_handle, qsq.query_hash, qsp.query_plan_hash, qst.query_sql_text, qsp.query_plan
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id
JOIN sys.query_store_plan qsp ON qsq.query_id = qsp.query_id
JOIN (
	SELECT query_hash AS [query_hash], COUNT(query_id) AS [Count]
	FROM sys.query_store_query 
	GROUP BY query_hash
	HAVING COUNT(*) > 1
	) m on qsq.query_hash = m.query_hash
WHERE qst.query_sql_text like '%color%'



USE [master];
GO
ALTER DATABASE [AdventureWorks2016] SET COMPATIBILITY_LEVEL = 110
GO

USE [AdventureWorks2016];
GO

SELECT [od].[SalesOrderID], [od].[SalesOrderDetailID]
FROM Sales.[SalesOrderDetail] AS [od]
INNER JOIN Production.[Product] AS [p] 
 ON [od].[ProductID] = [p].[ProductID]
WHERE [p].[Color] = 'Red' AND
[od].[ModifiedDate] = '2015-06-28 00:00:00.000';


USE [master];
GO
ALTER DATABASE [AdventureWorks2016] SET COMPATIBILITY_LEVEL = 130
GO


USE [AdventureWorks2016];
GO

EXEC sp_query_store_flush_db;

SELECT 
	qst.query_text_id, qsq.query_id, qsp.plan_id, qsq.is_internal_query, qsq.object_id, 
	qsq.count_compiles, qsp.engine_version, qsp.compatibility_level, qsp.is_parallel_plan, 
	qsp.is_forced_plan,	qsp.is_trivial_plan, qsp.last_compile_duration, qsp.last_execution_time,
	qst.statement_sql_handle, qsq.query_hash, qsp.query_plan_hash, qst.query_sql_text, qsp.query_plan
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id
JOIN sys.query_store_plan qsp ON qsq.query_id = qsp.query_id
WHERE qsp.compatibility_level < 120;


SELECT 
	qst.query_text_id, qsq.query_id, qsp.plan_id, qsq.is_internal_query, qsq.object_id, 
	qsq.count_compiles, qsp.engine_version, qsp.compatibility_level, qsp.is_parallel_plan, 
	qsp.is_forced_plan,	qsp.is_trivial_plan, qsp.last_compile_duration, qsp.last_execution_time,
	qst.statement_sql_handle, qsq.query_hash, qsp.query_plan_hash, qst.query_sql_text, qsp.query_plan
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id
JOIN sys.query_store_plan qsp ON qsq.query_id = qsp.query_id
ORDER BY qst.query_sql_text


EXEC sp_query_store_remove_query @query_id = 56;

SELECT [od].[SalesOrderID], [od].[SalesOrderDetailID]
FROM Sales.[SalesOrderDetail] AS [od]
INNER JOIN Production.[Product] AS [p] 
 ON [od].[ProductID] = [p].[ProductID]
WHERE [p].[Color] = 'Red' AND
[od].[ModifiedDate] = '2015-06-28 00:00:00.000'
OPTION (QUERYTRACEON 9481) -- revert to 2012 CE

SELECT [od].[SalesOrderID], [od].[SalesOrderDetailID]
FROM Sales.[SalesOrderDetail] AS [od]
INNER JOIN Production.[Product] AS [p] 
 ON [od].[ProductID] = [p].[ProductID]
WHERE [p].[Color] = 'Red' AND
[od].[ModifiedDate] = '2015-06-28 00:00:00.000'




/*
	turn it off...
*/
USE [master];
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE = OFF;
GO

/*
	everything is still in QS views
*/
USE [AdventureWorks2016];
GO

SELECT 
	qst.query_text_id, qsq.query_id, qsq.is_internal_query, qsq.object_id, 
	qsq.count_compiles, qst.query_sql_text, qst.statement_sql_handle, qsq.query_hash
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id;


/*
	clear what's in the query store
*/
--ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE CLEAR;



sp_configure 'remote admin connections', 1

reconfigure
GO


USE [AdventureWorks2016E];
GO


SELECT *
FROM [sys].[plan_persist_query_text];
GO

SELECT *
FROM [sys].[plan_persist_context_settings];
GO

SELECT *
FROM [sys].[plan_persist_plan];
GO

SELECT *
FROM [sys].[plan_persist_query];
GO

SELECT *
FROM [sys].[plan_persist_runtime_stats];
GO

SELECT *
FROM [sys].[plan_persist_runtime_stats_interval];
GO




