/*
	check existing SP definition
*/
USE [AdventureWorks2016E];
GO

SELECT OBJECT_DEFINITION (OBJECT_ID(N'Sales.usp_GetFullProductInfo'))

  CREATE PROCEDURE [Sales].[usp_GetFullProductInfo]   @ProductID INT  AS      

  SELECT [he].[CustomerID], [he].[OrderDate], [de].[ProductID], [he].[SubTotal]   
  FROM [Sales].[SalesOrderDetailEnlarged] [de]   
  JOIN [Sales].[SalesOrderHeaderEnlarged] [he] on [de].[SalesOrderID] = [he].[SalesOrderID]   
  WHERE [ProductID] = @ProductID   
  ORDER BY [ProductID];    

/*
	check indexes which exist
*/
sp_SQLskills_SQL2012_helpindex 'Sales.SalesOrderDetailEnlarged'


/*
	find the query_id in QS and check the plan

*/
SELECT 
	qst.query_text_id, qsq.query_id, qsq.object_id, 
	qsq.count_compiles, qst.query_sql_text, qst.statement_sql_handle, qsq.query_hash
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id
WHERE qst.query_sql_text like '%SalesOrderDetailEnlarged%'


/*
	I was going to change the query, so it wouldn't have an index to support it
	but...
*/
DROP PROCEDURE IF EXISTS [Sales].[usp_GetFullProductInfo];
GO

CREATE PROCEDURE [Sales].[usp_GetFullProductInfo]
@ProductID INT
AS	
SELECT [he].[CustomerID], [he].[OrderDate], [de].[ProductID], [de].[OrderQty], [he].[SubTotal]
FROM [Sales].[SalesOrderDetailEnlarged] [de]
JOIN [Sales].[SalesOrderHeaderEnlarged] [he] on [de].[SalesOrderID] = [he].[SalesOrderID]
WHERE [de].[ProductID] = 890
ORDER BY [de].[ProductID];

/*
	it changed the query text!!  
	:)
	Plan B

	run other PS
	check indexes
*/
EXEC sp_SQLskills_SQL2012_helpindex 'Sales.SalesOrderDetailEnlarged'

/*
	check the SP now
	force the plan
*/


/*
	check status of forced plan
*/
SELECT p.plan_id, p.query_id, q.object_id as containing_object_id,
    force_failure_count, last_force_failure_reason_desc
FROM sys.query_store_plan AS p
JOIN sys.query_store_query AS q on p.query_id = q.query_id
WHERE is_forced_plan = 1;


/*
	what's in cache now?
*/
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
GO
SELECT qs.execution_count, s.text, qs.query_hash, qs.query_plan_hash, cp.size_in_bytes/1024 [PlanSizeKB], qp.query_plan, qs.plan_handle
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) qp
CROSS APPLY sys.dm_exec_sql_text(qs.plan_handle) s
JOIN sys.dm_exec_cached_plans cp ON qs.plan_handle = cp.plan_handle
WHERE s.text like '%SalesOrderDetailEnlarged%';
GO
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;
GO
